@@ -18,10 +18,13 @@ import uuid
 import warnings
 
 from datetime import datetime
+from functools import wraps
+from types import FunctionType
 
 import raven
 from raven.conf import defaults
 from raven.context import Context
+from raven.exceptions import APIError, RateLimited
 from raven.utils import six, json, get_versions, get_auth_header, merge_dicts
 from raven.utils.encoding import to_unicode
 from raven.utils.serializer import transform
@@ -58,27 +61,30 @@ class ClientState(object):
         self.status = self.ONLINE
         self.last_check = None
         self.retry_number = 0
+        self.retry_after = 0
 
     def should_try(self):
         if self.status == self.ONLINE:
             return True
 
-        interval = min(self.retry_number, 6) ** 2
+        interval = self.retry_after or min(self.retry_number, 6) ** 2
 
         if time.time() - self.last_check > interval:
             return True
 
         return False
 
-    def set_fail(self):
+    def set_fail(self, retry_after=0):
         self.status = self.ERROR
         self.retry_number += 1
         self.last_check = time.time()
+        self.retry_after = retry_after
 
     def set_success(self):
         self.status = self.ONLINE
         self.last_check = None
         self.retry_number = 0
+        self.retry_after = 0
 
     def did_fail(self):
         return self.status == self.ERROR
@@ -270,7 +276,6 @@ class Client(object):
 
         data.setdefault('tags', {})
         data.setdefault('extra', {})
-        data.setdefault('level', logging.ERROR)
 
         if stack is None:
             stack = self.auto_log_stacks
@@ -503,7 +508,7 @@ class Client(object):
         # decode message so we can show the actual event
         try:
             data = self.decode(data)
-        except:
+        except Exception:
             message = '<failed decoding data>'
         else:
             message = data.pop('message', '<no message value>')
@@ -520,7 +525,12 @@ class Client(object):
         self.state.set_success()
 
     def _failed_send(self, e, url, data):
-        if isinstance(e, HTTPError):
+        retry_after = 0
+        if isinstance(e, APIError):
+            if isinstance(e, RateLimited):
+                retry_after = e.retry_after
+            self.error_logger.error('Unable to capture event: %s', e.message)
+        elif isinstance(e, HTTPError):
             body = e.read()
             self.error_logger.error(
                 'Unable to reach Sentry log server: %s (url: %s, body: %s)',
@@ -533,7 +543,7 @@ class Client(object):
 
         message = self._get_log_message(data)
         self.error_logger.error('Failed to submit message: %r', message)
-        self.state.set_fail()
+        self.state.set_fail(retry_after=retry_after)
 
     def send_remote(self, url, data, headers=None):
         if headers is None:
@@ -635,6 +645,40 @@ class Client(object):
         return self.capture(
             'raven.events.Exception', exc_info=exc_info, **kwargs)
 
+    def capture_exceptions(self, function_or_exceptions, **kwargs):
+        """
+        Wrap a function in try/except and automatically call ``.captureException``
+        if it raises an exception, then the exception is reraised.
+
+        By default, it will capture ``Exception``
+
+        >>> @client.capture_exceptions
+        >>> def foo():
+        >>>     raise Exception()
+
+        You can also specify exceptions to be caught specifically
+
+        >>> @client.capture_exceptions((IOError, LookupError))
+        >>> def bar():
+        >>>     ...
+
+        ``kwargs`` are passed through to ``.captureException``.
+        """
+        def make_decorator(exceptions):
+            def decorator(func):
+                @wraps(func)
+                def wrapper(*funcargs, **funckwargs):
+                    try:
+                        return func(*funcargs, **funckwargs)
+                    except exceptions:
+                        self.captureException(**kwargs)
+                        raise
+                return wrapper
+            return decorator
+        if isinstance(function_or_exceptions, FunctionType):
+            return make_decorator((Exception,))(function_or_exceptions)
+        return make_decorator(function_or_exceptions)
+
     def captureQuery(self, query, params=(), engine=None, **kwargs):
         """
         Creates an event for a SQL query.
