@@ -14,6 +14,12 @@ except ImportError:
     from urllib2 import HTTPError  # NOQA
 
 
+try:
+    import httplib  # NOQA
+except ImportError:
+    from http import client as httplib  # NOQA
+
+
 try:
     import urllib.request as urllib2
 except ImportError:
